@@ -44,6 +44,7 @@ html_logo = "_static/logo.svg"
 # extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
 # ones.
 extensions = [
+    "sphinx_copybutton",
     "sphinx.ext.autodoc",
     "sphinx.ext.autosummary",
     "sphinx.ext.doctest",
@@ -56,6 +57,7 @@ extensions = [
     "sphinx_autodoc_typehints",
     "nbsphinx",
     "IPython.sphinxext.ipython_console_highlighting",
+    "sphinxcontrib.mermaid",
 ]
 
 
