@@ -1,21 +1,13 @@
 from __future__ import annotations
 
-import copy
 import hashlib
-from contextlib import contextmanager
 from typing import Any, Sequence
 
 import jax
 import jax.tree_util as jtu
 import numpy as np
 
-from pytreeclass._src.tree_decorator import (
-    _FROZEN,
-    _VARS,
-    _WRAPPED,
-    fields,
-    is_treeclass,
-)
+_WRAPPED = "__wrapped__"
 
 PyTree = Any
 
@@ -32,15 +24,11 @@ def _hash_node(node: Any) -> int:
     return hash(node)
 
 
-def _tree_hash(tree: PyTree) -> int:
+def tree_hash(tree: PyTree) -> int:
     hashed = jtu.tree_map(_hash_node, jtu.tree_leaves(tree))
     return hash((*hashed, jtu.tree_structure(tree)))
 
 
-def _unwrap(value: Any) -> Any:
-    return value.unwrap() if isinstance(value, ImmutableWrapper) else value
-
-
 class ImmutableWrapper:
     """Base class for all immutable wrappers that gets a special treatment inside `treeclass` wrapped classes.
     In essence, this wrapper is rendered transparent inside `treeclass` wrapped classes
@@ -78,7 +66,7 @@ class ImmutableWrapper:
 
     def __init__(self, x: Any) -> None:
         # disable composition of Wrappers
-        getattr(self, _VARS)[_WRAPPED] = _unwrap(x)
+        vars(self)[_WRAPPED] = x.unwrap() if isinstance(x, ImmutableWrapper) else x
 
     def unwrap(self) -> Any:
         return getattr(self, _WRAPPED)
@@ -90,14 +78,6 @@ class ImmutableWrapper:
         raise AttributeError("Cannot delete from frozen instance.")
 
 
-def _tree_unwrap(value: PyTree) -> PyTree:
-    # enables the transparent wrapper behavior iniside `treeclass` wrapped classes
-    def is_leaf(x: Any) -> bool:
-        return isinstance(x, ImmutableWrapper) or is_treeclass(x)
-
-    return jtu.tree_map(_unwrap, value, is_leaf=is_leaf)
-
-
 class _HashableWrapper(ImmutableWrapper):
     # used to wrap metadata to make it hashable
     # this is intended to wrap frozen values to avoid error when comparing
@@ -130,7 +110,7 @@ def _frozen_flatten(tree: Any) -> tuple[tuple, Any]:
 
 def _frozen_unflatten(treedef: Any, _: Sequence[Any]) -> PyTree:
     tree = object.__new__(FrozenWrapper)  # type: ignore
-    getattr(tree, _VARS)[_WRAPPED] = treedef.unwrap()
+    vars(tree)[_WRAPPED] = treedef.unwrap()
     return tree
 
 
@@ -240,34 +220,3 @@ def is_nondiff(x: Any) -> bool:
     if isinstance(x, (float, complex)):
         return False
     return True
-
-
-@contextmanager
-def _call_context(tree: PyTree):
-    def mutate_step(tree: PyTree):
-        if not is_treeclass(tree):
-            return tree
-        # shadow the class _FROZEN attribute with an
-        # instance variable to temporarily disable the frozen behavior
-        # after the context manager exits, the instance variable will be deleted
-        # and the class attribute will be used again.
-        getattr(tree, _VARS)[_FROZEN] = False
-        for field in fields(tree):
-            mutate_step(getattr(tree, field.name))
-        return tree
-
-    def immutate_step(tree):
-        if not is_treeclass(tree):
-            return tree
-        if _FROZEN not in getattr(tree, _VARS):
-            return tree
-
-        del getattr(tree, _VARS)[_FROZEN]
-        for field in fields(tree):
-            immutate_step(getattr(tree, field.name))
-        return tree
-
-    tree = copy.copy(tree)
-    mutate_step(tree)
-    yield tree
-    immutate_step(tree)
