@@ -605,7 +605,35 @@ def tree_mermaid(
     #     generated_html = f"https://pytreeclass.herokuapp.com/temp/?id={generated_id}"
     #     return f"Open URL in browser: {generated_html}"
 
-    """generate a mermaid diagram syntax for arbitrary PyTrees."""
+    """generate a mermaid diagram syntax for arbitrary PyTrees.
+
+
+    Args:
+        tree: PyTree
+        width: max width of line. default is 60
+        depth: depth of the tree to print. default is max depth
+        is_leaf: function to determine if a node is a leaf. default is None
+
+    Example:
+        >>> import pytreeclass as pytc
+        >>> tree = [1, [2, 3], [4, [5, 6]]]
+        >>> print(pytc.tree_mermaid(tree, depth=1))  # doctest: +SKIP
+
+        .. mermaid::
+
+            flowchart LR
+
+                id2 --- id3("</b>[0]=2</b>")
+                id2 --- id4("</b>[1]=3</b>")
+                id5 --- id6("</b>[0]=4</b>")
+                id5 --- id7("</b>[1]:list</b>")
+                id0(<b>list</b>)
+                id0 --- id1("</b>[0]=1</b>")
+                id0 --- id2("</b>[1]:list</b>")
+                id0 --- id5("</b>[2]:list</b>")
+                id7 --- id8("</b>[0]=5</b>")
+                id7 --- id9("</b>[1]=6</b>")
+    """
 
     indent_repr = tree_indent(
         tree,
