@@ -1,7 +1,6 @@
 import copy
 import dataclasses as dc
 import functools as ft
-from typing import Tuple
 
 import jax
 import jax.tree_util as jtu
@@ -10,12 +9,12 @@ import pytest
 from jax import numpy as jnp
 
 import pytreeclass as pytc
-from pytreeclass._src.tree_decorator import _FIELD_MAP, _dataclass_like_fields, fields
+from pytreeclass._src.tree_decorator import _field_registry
 
 
 def test_field():
     with pytest.raises(ValueError):
-        pytc.field(default=1, default_factory=lambda: 1)
+        pytc.field(default=1, factory=lambda: 1)
 
     assert pytc.field(default=1).default == 1
 
@@ -26,7 +25,7 @@ def test_field():
     class Test:
         a: int = pytc.field(default=1, metadata={"a": 1})
 
-    assert getattr(Test(), _FIELD_MAP)["a"].metadata["a"] == 1
+    assert _field_registry[Test]["a"].metadata["a"] == 1
 
     @ft.partial(pytc.treeclass, leafwise=True)
     class Test:
@@ -93,6 +92,22 @@ def test_field():
 
     assert Test(b=2).a == 1
 
+    with pytest.raises(TypeError):
+        pytc.fields(1)
+
+    assert len(pytc.fields(Test)) == 2
+
+    @pytc.treeclass
+    class Test:
+        a: int = pytc.field(default=1)
+        b: int = pytc.field(factory=lambda: 1)
+
+        def __init__(self) -> None:
+            pass
+
+    with pytest.raises(AttributeError):
+        Test()
+
 
 def test_field_nondiff():
     @ft.partial(pytc.treeclass, leafwise=True)
@@ -353,6 +368,12 @@ def test_mutable_field():
         class Test:
             a: list = [1, 2, 3]
 
+    with pytest.raises(TypeError):
+
+        @ft.partial(pytc.treeclass, leafwise=True)
+        class Test:
+            a: list = pytc.field(default=[1, 2, 3])
+
 
 def test_non_class_input():
     with pytest.raises(TypeError):
@@ -363,14 +384,14 @@ def test_non_class_input():
 
 
 def test_setattr_delattr():
-    with pytest.raises(AttributeError):
+    with pytest.raises(TypeError):
 
         @ft.partial(pytc.treeclass, leafwise=True)
         class Test:
             def __setattr__(self, k, v):
                 pass
 
-    with pytest.raises(AttributeError):
+    with pytest.raises(TypeError):
 
         @ft.partial(pytc.treeclass, leafwise=True)
         class _:
@@ -454,6 +475,8 @@ def test_callbacks():
         class Test:
             a: int = pytc.field(callbacks=[lambda: True])
 
+        Test(a=1)
+
 
 def test_treeclass_frozen_field():
     @ft.partial(pytc.treeclass, leafwise=True)
@@ -478,20 +501,6 @@ def test_key_error():
         Test()
 
 
-def test_dataclass_fields_like():
-    @dc.dataclass
-    class Test:
-        a: int = 1
-
-    assert _dataclass_like_fields(Test) == dc.fields(Test)
-
-    with pytest.raises(TypeError):
-        _dataclass_like_fields(1)
-
-    with pytest.raises(TypeError):
-        fields(1)
-
-
 def test_super():
     @ft.partial(pytc.treeclass, leafwise=True)
     class Test:
@@ -540,21 +549,21 @@ def test_optional_param():
 
 
 def benchmark_dataclass_class():
-    @dc.dataclass
-    class Test:
-        a: int = 1
-        b: Tuple[int, ...] = (1, 2, 3)
-
-    return Test()
+    count = 10
+    annot = {f"leaf_{i}": int for i in range(count)}
+    leaves = {f"leaf_{i}": i for i in range(count)}
+    Tree = type("Tree", (), {"__annotations__": annot, **leaves})
+    Tree = dc.dataclass(Tree)
+    return Tree()
 
 
 def benchmark_treeclass_instance():
-    @pytc.treeclass
-    class Test:
-        a: int = 1
-        b: Tuple[int, ...] = (1, 2, 3)
-
-    return Test(1, (1, 2, 3))
+    count = 10
+    annot = {f"leaf_{i}": int for i in range(count)}
+    leaves = {f"leaf_{i}": i for i in range(count)}
+    Tree = type("Tree", (), {"__annotations__": annot, **leaves})
+    Tree = pytc.treeclass(Tree)
+    return Tree()
 
 
 @pytest.mark.benchmark(group="treeclass")
@@ -698,3 +707,20 @@ def test_incorrect_trace_func():
     pytc.tree_leaves_with_trace(T())
 
     assert True
+
+
+def test_self_field_name():
+    with pytest.raises(ValueError):
+
+        @pytc.treeclass
+        class Tree:
+            self: int = pytc.field()
+
+
+def test_repeated_treeclass():
+    @pytc.treeclass
+    @pytc.treeclass
+    class Tree:
+        a: int = pytc.field()
+
+    assert True
